/*
 * PHP-To-AST - PHP parser
 * Copyright (c) Dan Phillimore (asmblah)
 * http://uniter.github.com/phptoast/
 *
 * Released under the MIT license
 * https://github.com/uniter/phptoast/raw/master/MIT-LICENSE.txt
 */

'use strict';

var _ = require('microdash'),
    expect = require('chai').expect,
    nowdoc = require('nowdoc'),
    tools = require('../../tools');

describe('PHP Parser grammar class statement integration', function () {
    var parser;

    beforeEach(function () {
        parser = tools.createParser();
    });

    _.each({
        'empty class that does not extend or implement': {
            code: '<?php class Test {}',
            expectedAST: {
                name: 'N_PROGRAM',
                statements: [{
                    name: 'N_CLASS_STATEMENT',
                    className: 'Test',
                    members: []
                }]
            }
        },
        'class with one public instance property with no value': {
            code: nowdoc(function () {/*<<<EOS
<?php
    class OnePub {
        public $aPublicProp;
    }
EOS
*/;}), // jshint ignore:line
            expectedAST: {
                name: 'N_PROGRAM',
                statements: [{
                    name: 'N_CLASS_STATEMENT',
                    className: 'OnePub',
                    members: [{
                        name: 'N_INSTANCE_PROPERTY_DEFINITION',
                        visibility: 'public',
                        variable: {
                            name: 'N_VARIABLE',
                            variable: 'aPublicProp'
                        }
                    }]
                }]
            }
        },
        'class with one public instance property with string value': {
            code: nowdoc(function () {/*<<<EOS
<?php
    class OnePub {
        public $aPublicProp = 'yep';
    }
EOS
*/;}), // jshint ignore:line
            expectedAST: {
                name: 'N_PROGRAM',
                statements: [{
                    name: 'N_CLASS_STATEMENT',
                    className: 'OnePub',
                    members: [{
                        name: 'N_INSTANCE_PROPERTY_DEFINITION',
                        visibility: 'public',
                        variable: {
                            name: 'N_VARIABLE',
                            variable: 'aPublicProp'
                        },
                        value: {
                            name: 'N_STRING_LITERAL',
                            string: 'yep'
                        }
                    }]
                }]
            }
        },
        'class with one public instance method': {
            code: nowdoc(function () {/*<<<EOS
<?php
    class OneMethod {
        public function printIt($what) {
            echo $what;
        }
    }
EOS
*/;}), // jshint ignore:line
            expectedAST: {
                name: 'N_PROGRAM',
                statements: [{
                    name: 'N_CLASS_STATEMENT',
                    className: 'OneMethod',
                    members: [{
                        name: 'N_METHOD_DEFINITION',
                        visibility: 'public',
                        func: {
                            name: 'N_STRING',
                            string: 'printIt'
                        },
                        args: [{
                            name: 'N_ARGUMENT',
                            variable: {
                                name: 'N_VARIABLE',
                                variable: 'what'
                            }
                        }],
                        body: {
                            name: 'N_COMPOUND_STATEMENT',
                            statements: [{
                                name: 'N_ECHO_STATEMENT',
                                expressions: [{
                                    name: 'N_VARIABLE',
                                    variable: 'what'
                                }]
                            }]
                        }
                    }]
                }]
            }
        },
        'class with one public instance method with one body statement not wrapped in braces': {
            code: nowdoc(function () {/*<<<EOS
<?php
    class OneMethod {
        public function printIt($what) echo $what;
    }
EOS
*/;}), // jshint ignore:line
            expectedAST: {
                name: 'N_PROGRAM',
                statements: [{
                    name: 'N_CLASS_STATEMENT',
                    className: 'OneMethod',
                    members: [{
                        name: 'N_METHOD_DEFINITION',
                        visibility: 'public',
                        func: {
                            name: 'N_STRING',
                            string: 'printIt'
                        },
                        args: [{
                            name: 'N_ARGUMENT',
                            variable: {
                                name: 'N_VARIABLE',
                                variable: 'what'
                            }
                        }],
                        body: {
                            name: 'N_ECHO_STATEMENT',
                            expressions: [{
                                name: 'N_VARIABLE',
                                variable: 'what'
                            }]
                        }
                    }]
                }]
            }
        },
        'class with one implicitly public instance method': {
            code: nowdoc(function () {/*<<<EOS
<?php
class OneMethod {
    function printIt($msg) echo $msg;
}
EOS
*/;}), // jshint ignore:line
            expectedAST: {
                name: 'N_PROGRAM',
                statements: [{
                    name: 'N_CLASS_STATEMENT',
                    className: 'OneMethod',
                    members: [{
                        name: 'N_METHOD_DEFINITION',
                        visibility: 'public',
                        func: {
                            name: 'N_STRING',
                            string: 'printIt'
                        },
                        args: [{
                            name: 'N_ARGUMENT',
                            variable: {
                                name: 'N_VARIABLE',
                                variable: 'msg'
                            }
                        }],
                        body: {
                            name: 'N_ECHO_STATEMENT',
                            expressions: [{
                                name: 'N_VARIABLE',
                                variable: 'msg'
                            }]
                        }
                    }]
                }]
            }
        }
    }, function (scenario, description) {
        describe(description, function () {
            // Pretty-print the code strings so any non-printable characters are readable
            describe('when the code is ' + JSON.stringify(scenario.code) + ' ?>', function () {
                it('should return the expected AST', function () {
                    expect(parser.parse(scenario.code)).to.deep.equal(scenario.expectedAST);
                });
            });
        });
    });
});
